@@ -47,7 +47,7 @@ Parse Plist example:
 """
 __all__ = [
     "readPlist", "writePlist", "readPlistFromBytes", "writePlistToBytes",
-    "Plist", "Data", "Dict", "FMT_XML", "FMT_BINARY",
+    "Data", "InvalidFileException", "FMT_XML", "FMT_BINARY",
     "load", "dump", "loads", "dumps"
 ]
 
@@ -76,44 +76,6 @@ globals().update(PlistFormat.__members__)
 #
 
 
-class _InternalDict(dict):
-
-    # This class is needed while Dict is scheduled for deprecation:
-    # we only need to warn when a *user* instantiates Dict or when
-    # the "attribute notation for dict keys" is used.
-    __slots__ = ()
-
-    def __getattr__(self, attr):
-        try:
-            value = self[attr]
-        except KeyError:
-            raise AttributeError(attr)
-        warn("Attribute access from plist dicts is deprecated, use d[key] "
-             "notation instead", DeprecationWarning, 2)
-        return value
-
-    def __setattr__(self, attr, value):
-        warn("Attribute access from plist dicts is deprecated, use d[key] "
-             "notation instead", DeprecationWarning, 2)
-        self[attr] = value
-
-    def __delattr__(self, attr):
-        try:
-            del self[attr]
-        except KeyError:
-            raise AttributeError(attr)
-        warn("Attribute access from plist dicts is deprecated, use d[key] "
-             "notation instead", DeprecationWarning, 2)
-
-
-class Dict(_InternalDict):
-
-    def __init__(self, **kwargs):
-        warn("The plistlib.Dict class is deprecated, use builtin dict instead",
-             DeprecationWarning, 2)
-        super().__init__(**kwargs)
-
-
 @contextlib.contextmanager
 def _maybe_open(pathOrFile, mode):
     if isinstance(pathOrFile, str):
@@ -124,31 +86,6 @@ def _maybe_open(pathOrFile, mode):
         yield pathOrFile
 
 
-class Plist(_InternalDict):
-    """This class has been deprecated. Use dump() and load()
-    functions instead, together with regular dict objects.
-    """
-
-    def __init__(self, **kwargs):
-        warn("The Plist class is deprecated, use the load() and "
-             "dump() functions instead", DeprecationWarning, 2)
-        super().__init__(**kwargs)
-
-    @classmethod
-    def fromFile(cls, pathOrFile):
-        """Deprecated. Use the load() function instead."""
-        with _maybe_open(pathOrFile, 'rb') as fp:
-            value = load(fp)
-        plist = cls()
-        plist.update(value)
-        return plist
-
-    def write(self, pathOrFile):
-        """Deprecated. Use the dump() function instead."""
-        with _maybe_open(pathOrFile, 'wb') as fp:
-            dump(self, fp)
-
-
 def readPlist(pathOrFile):
     """
     Read a .plist from a path or file. pathOrFile should either
@@ -160,8 +97,7 @@ def readPlist(pathOrFile):
         DeprecationWarning, 2)
 
     with _maybe_open(pathOrFile, 'rb') as fp:
-        return load(fp, fmt=None, use_builtin_types=False,
-            dict_type=_InternalDict)
+        return load(fp, fmt=None, use_builtin_types=False)
 
 def writePlist(value, pathOrFile):
     """
@@ -184,8 +120,7 @@ def readPlistFromBytes(data):
     """
     warn("The readPlistFromBytes function is deprecated, use loads() instead",
         DeprecationWarning, 2)
-    return load(BytesIO(data), fmt=None, use_builtin_types=False,
-        dict_type=_InternalDict)
+    return load(BytesIO(data), fmt=None, use_builtin_types=False)
 
 
 def writePlistToBytes(value):
@@ -225,10 +160,10 @@ class Data:
     def __eq__(self, other):
         if isinstance(other, self.__class__):
             return self.data == other.data
-        elif isinstance(other, str):
+        elif isinstance(other, bytes):
             return self.data == other
         else:
-            return id(self) == id(other)
+            return NotImplemented
 
     def __repr__(self):
         return "%s(%s)" % (self.__class__.__name__, repr(self.data))
@@ -625,7 +560,8 @@ class _BinaryPlistParser:
             self._objects = [_undefined] * num_objects
             return self._read_object(top_object)
 
-        except (OSError, IndexError, struct.error):
+        except (OSError, IndexError, struct.error, OverflowError,
+                UnicodeDecodeError):
             raise InvalidFileException()
 
     def _get_size(self, tokenL):
@@ -643,6 +579,8 @@ class _BinaryPlistParser:
         if size in _BINARY_FORMAT:
             return struct.unpack('>' + _BINARY_FORMAT[size] * n, data)
         else:
+            if not size or len(data) != size * n:
+                raise InvalidFileException()
             return tuple(int.from_bytes(data[i: i + size], 'big')
                          for i in range(0, size * n, size))
 
@@ -693,7 +631,8 @@ class _BinaryPlistParser:
             f = struct.unpack('>d', self._fp.read(8))[0]
             # timestamp 0 of binary plists corresponds to 1/1/2001
             # (year of Mac OS X 10.0), instead of 1/1/1970.
-            result = datetime.datetime.utcfromtimestamp(f + (31 * 365 + 8) * 86400)
+            result = (datetime.datetime(2001, 1, 1) +
+                      datetime.timedelta(seconds=f))
 
         elif tokenH == 0x40:  # data
             s = self._get_size(tokenL)
@@ -932,7 +871,7 @@ class _BinaryPlistWriter (object):
                 self._write_size(0x50, len(value))
             except UnicodeEncodeError:
                 t = value.encode('utf-16be')
-                self._write_size(0x60, len(value))
+                self._write_size(0x60, len(t) // 2)
 
             self._fp.write(t)
 
